// This would be nice to get rid of...
protocol Multipliable {
    func * (lhs: Self, rhs: Self) -> Self
}

extension Float : Multipliable { }
extension Double : Multipliable { }
extension Float80 : Multipliable { }
// End nice-to-get-rid-of stuff.

struct FloatingPointStrideToGenerator<Element : Strideable where Element.Stride : FloatingPointType, Element.Stride : Multipliable> : GeneratorType {
    let start: Element
    let end: Element
    let stride: Element.Stride
    var iteration: Int = 0
    
    /// Advance to the next element and return it, or `nil` if no next element exists.
    mutating func next() -> Element? {
        let current = start + Element.Stride(iteration) * stride
        if stride > 0 ? current >= end : current <= end {
            return nil
        }
        iteration += 1
        return current
    }
    
    init(_start: Element, _end: Element, _stride: Element.Stride) {
        self.start = _start
        self.end = _end
        self.stride = _stride
    }
}

struct FloatingPointStrideTo<Element : Strideable where Element.Stride : FloatingPointType, Element.Stride : Multipliable> : SequenceType, CustomReflectable {
    func generate() -> FloatingPointStrideToGenerator<Element> {
        return FloatingPointStrideToGenerator(_start: _start, _end: _end, _stride: _stride)
    }
    
    init(_start: Element, end: Element, stride: Element.Stride) {
        precondition(stride != 0, "stride size must not be zero")
        self._start = _start
        self._end = end
        self._stride = stride
    }
    
    let _start: Element
    let _end: Element
    let _stride: Element.Stride
    
    func customMirror() -> Mirror {
        return Mirror(self, children: ["from": _start, "to": _end, "by": _stride])
    }
}

struct IntegerStrideToGenerator<Element : Strideable where Element.Stride : IntegerType> : GeneratorType {
    var _current: Element
    let _end: Element
    let _stride: Element.Stride
    
    /// Advance to the next element and return it, or `nil` if no next element exists.
    mutating func next() -> Element? {
        if _stride > 0 ? _current >= _end : _current <= _end {
            return nil
        }
        let result = _current
        _current += _stride
        return result
    }
    
    init(_start: Element, _end: Element, _stride: Element.Stride) {
        self._current = _start
        self._end = _end
        self._stride = _stride
    }
}

struct IntegerStrideTo<Element : Strideable where Element.Stride : IntegerType> : SequenceType, CustomReflectable {
    func generate() -> IntegerStrideToGenerator<Element> {
        return IntegerStrideToGenerator(_start: _start, _end: _end, _stride: _stride)
    }
    
    init(_start: Element, end: Element, stride: Element.Stride) {
        precondition(stride != 0, "stride size must not be zero")
        self._start = _start
        self._end = end
        self._stride = stride
    }
    
    let _start: Element
    let _end: Element
    let _stride: Element.Stride
    
    func customMirror() -> Mirror {
        return Mirror(self, children: ["from": _start, "to": _end, "by": _stride])
    }
}

extension HalfOpenInterval where Bound : Strideable, Bound.Stride : FloatingPointType, Bound.Stride : Multipliable {
    func striding(by stride: Bound.Stride) -> FloatingPointStrideTo<Bound> {
        return FloatingPointStrideTo(_start: start, end: end, stride: stride)
    }
}

extension Range where Element : Strideable, Element.Stride : IntegerType {
    func striding(by stride: Element.Stride) -> IntegerStrideTo<Element> {
        return IntegerStrideTo(_start: startIndex, end: endIndex, stride: stride)
    }
}

@warn_unused_result
func stride<T : Strideable where T.Stride : FloatingPointType, T.Stride : Multipliable>(from start: T, to end: T, by stride: T.Stride) -> FloatingPointStrideTo<T> {
    return FloatingPointStrideTo(_start: start, end: end, stride: stride)
}

@warn_unused_result
func stride<T : Strideable where T.Stride : IntegerType>(from start: T, to end: T, by stride: T.Stride) -> IntegerStrideTo<T> {
    return IntegerStrideTo(_start: start, end: end, stride: stride)
}

print(Array((1.0..<10.0).striding(by: 2.0)))
print(Array((1..<10).striding(by: 2)))
print(Array(stride(from: 0.0, to: -10.0, by: -1.0)))
print(Array(stride(from: 0, to: -10, by: -1)))

// Look, we're not accumulating error!
print(Array((1.0..<2.0).striding(by: 0.1)))
